#ifndef __H_CALLBACK__
#define __H_CALLBACK__

#include "baseobject.h"
#include "defs.h"
#include "Func.h"

BEGIN_DECLS

/**
 * @file
 * @defgroup SCallback SCallback
 * @addtogroup SCallback
 * @{
 * Callbacks are used to signals one part of the a program to an other.
 * It is manly used in SObject s and its children.
 *
 * Please note that this is the best way to implement an "interface".
 *
 * @sa GlobalNotify
 */

/**
 * An SCallbackEntry is used for convinience when installing callbacks into
 * SObjects. It may also be used in 
 */
typedef struct SCallbackEntry SCallbackEntry;

/**
 * The types for callbacks.
 */
typedef enum {
  S_CALLBACK_NULL,
  S_CALLBACK_CALLBACK,
  S_CALLBACK_NOTIFY,
  S_CALLBACK_NOTIFY_CHANGE
} SCallbackType;

/**
 * A list containing the names of the callback types.
 */
static char * SCallbackTypeNames[] __attribute__((unused)) = {
  "NULL",
  "CALLBACK",
  "NOTIFY",
  "NOTIFY_CHANGE",
  0x0,
  0x0,
};


struct SCallbackEntry {
  char * name; /**< The name that is used to invoke the callback. */
  SCallbackType type; /**< The type of the callback, is it a standard callback
                       * or a notify? */
  Callback callback; /**< The callback that will be invoked.*/
};



/**
 * @}
 * @addtogroup SObject
 * @{
 */

/**
 * Installs an array of callcackentries into an SObject.
 * 
 * @param obj The object to install the callback entries into.
 * @param callbackentries an array containing the callback entries.
 */
void s_object_install_callbacks (SObject * obj, SCallbackEntry ** callcackentries);

/**
 * Installs a singel callback into an SObject.
 *
 * @param obj The object to install the callback into.
 * @param callbackentry The callback entry to install.
 */
void s_object_install_callback (SObject * obj, SCallbackEntry * callbackentry);

/**
 * 
 */
spointer s_object_notify (SObject * self, char * name, spointer * data);

/**
 * Alias to s_object_notify()
 */
#define s_object_call(o, n, d) s_object_notify (o, n, d)

/**@}*/

END_DECLS

#endif
