/* c-basic-offset: 2; tab-width: 2; indent-tabs-mode: nil
 * vi: set shiftwidth=2 tabstop=2 expandtab:
 * :indentSize=2:tabSize=2:noTabs=true:
 */

/*
Copyright (c) 2013-2014 Gustav Hartvigsson

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

#include "DynamicArray.h"

struct SDynamicArrayPrivate {
  FreeFunc free_func;
  spointer * array;
};

void _private_for_each_item_free (SDynamicArray * self, spointer item,
                                    spointer data);

SDynamicArray * s_dynamic_array_new (size_t len, FreeFunc free_func) {
  SDynamicArray * self = malloc (sizeof(SDynamicArray));
  
  self->priv = malloc (sizeof (SDynamicArrayPrivate));
  
  self->max_size = len;
  
  self->priv->free_func = free_func;
  
  self->priv->array = calloc (len ,sizeof (* self->priv->array));
  
  return self;
}

void s_dynamic_array_free (SDynamicArray * self) {
  if (self->priv->free_func != NULL) {
    s_dynamic_array_for_each (self, (ForEachFunc) _private_for_each_item_free,
                            NULL);
  } else {
    for (int i = 0; i < self->last_item; i++) {
      free (self->priv->array[i]);
    }
  }
  free (self->priv->array);
  free (self);
}

spointer s_dynamic_array_get (SDynamicArray * self, size_t index) {
  return self->priv->array[index];
}

void s_dynamic_array_set (SDynamicArray * self, size_t index, spointer item) {
  if (self->max_size <= index) {
    size_t new_size = round_up (index + ARRAY_PADDING, ARRAY_PADDING) + 1;
    s_dbg_print ("Index: %d current array size: %d new array size: %d ",
      index, self->max_size, new_size);
    
    self->max_size = new_size;
    
    self->priv->array = realloc (self->priv->array,
                          (sizeof (* self->priv->array) * (self->max_size)));
  }
  
  if (self->last_item < index) {
    self->last_item = index;
  }
  
  self->priv->array[index] = item;
}

size_t s_dynamic_array_size (SDynamicArray * self) {
  return self->max_size;
}

size_t s_dynamic_array_last_item (SDynamicArray * self) {
  return self->last_item;
}

spointer * s_dynamic_array_dump_array (SDynamicArray * self) {
  spointer * ret_val = calloc (self->last_item + 1, sizeof (* self->priv->array));
  for (int i = 0; i < self->last_item; i++) {
    ret_val[i] = self->priv->array[i];
  }
  ret_val[self->last_item] = NULL;
  return ret_val;
}

void s_dynamic_array_for_each (SDynamicArray * self, ForEachFunc func,
                             spointer data) {
  for (int i = 0; i < self->max_size && i <= self->last_item; i++) {
    spointer item = self->priv->array[i];
    if (item) {
      func (self, item, data);
    }
  }
}


void _private_for_each_item_free (SDynamicArray * self, spointer item,
                                    spointer data) {
  FreeFunc func = self->priv->free_func;
  func (item);
}

